﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using System;
using System.Linq;

namespace HeuristicLab.Algorithms.OffspringSelectionCMAEvolutionStrategy {
  [Item("OffpsringSelectionCMATerminator", "Decides if the algorithm should terminate or not.")]
  [StorableClass]
  public class OffspringSelectionCMATerminator : HeuristicLab.Algorithms.CMAEvolutionStrategy.Terminator {

    public ILookupParameter<DoubleValue> SelectionPressureParameter {
      get { return (ILookupParameter<DoubleValue>)Parameters["SelectionPressure"]; }
    }

    public IValueLookupParameter<DoubleValue> MaximumSelectionPressureParameter {
      get { return (IValueLookupParameter<DoubleValue>)Parameters["MaximumSelectionPressure"]; }
    }

    [StorableConstructor]
    protected OffspringSelectionCMATerminator(bool deserializing) : base(deserializing) { }
    protected OffspringSelectionCMATerminator(OffspringSelectionCMATerminator original, Cloner cloner)
      : base(original, cloner) { }
    public OffspringSelectionCMATerminator() {
      Parameters.Add(new LookupParameter<DoubleValue>("SelectionPressure", "The current selection pressure."));
      Parameters.Add(new ValueLookupParameter<DoubleValue>("MaximumSelectionPressure", "The maximum selection pressure."));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new OffspringSelectionCMATerminator(this, cloner);
    }

    public override IOperation Apply() {
      var terminateOp = Terminate != null ? ExecutionContext.CreateOperation(Terminate) : null;

      var selPress = SelectionPressureParameter.ActualValue.Value;
      var maxSelPress = MaximumSelectionPressureParameter.ActualValue.Value;
      if (selPress >= maxSelPress) return terminateOp;

      return base.Apply();
    }
  }
}
